@@ -11,6 +11,10 @@ module Sequel
       TEMP_STORE = [:default, :file, :memory].freeze
       VIEWS_FILTER = "type = 'view'".freeze
 
+      # Whether to use integers for booleans in the database.  SQLite recommends
+      # booleans be stored as integers, but historically Sequel has used 't'/'f'.
+      attr_accessor :integer_booleans
+
       # A symbol signifying the value of the auto_vacuum PRAGMA.
       def auto_vacuum
         AUTO_VACUUM[pragma_get(:auto_vacuum).to_i]
@@ -89,6 +93,11 @@ module Sequel
       def pragma_set(name, value)
         execute_ddl("PRAGMA #{name} = #{value}")
       end
+
+      # Set the integer_booleans option using the passed in :integer_boolean option.
+      def set_integer_booleans
+        @integer_booleans = typecast_value_boolean(@opts[:integer_booleans])
+      end
       
       # The version of the server as an integer, where 3.6.19 = 30619.
       # If the server version can't be determined, 0 is used.
@@ -170,6 +179,8 @@ module Sequel
       # Run all alter_table commands in a transaction.  This is technically only
       # needed for drop column.
       def apply_alter_table(table, ops)
+        fks = foreign_keys
+        self.foreign_keys = false if fks
         transaction do 
           if ops.length > 1 && ops.all?{|op| op[:op] == :add_constraint}
             # If you are just doing constraints, apply all of them at the same time,
@@ -181,6 +192,7 @@ module Sequel
             ops.each{|op| alter_table_sql_list(table, [op]).flatten.each{|sql| execute_ddl(sql)}}
           end
         end
+        self.foreign_keys = true if fks
       end
 
       # SQLite supports limited table modification.  You can add a column
@@ -208,7 +220,7 @@ module Sequel
         when :set_column_null
           duplicate_table(table){|columns| columns.each{|s| s[:null] = op[:null] if s[:name].to_s == op[:name].to_s}}
         when :set_column_type
-          duplicate_table(table){|columns| columns.each{|s| s[:type] = op[:type] if s[:name].to_s == op[:name].to_s}}
+          duplicate_table(table){|columns| columns.each{|s| s.merge!(op) if s[:name].to_s == op[:name].to_s}}
         when :drop_constraint
           case op[:type]
           when :primary_key
@@ -238,6 +250,11 @@ module Sequel
         end
       end
 
+      # Surround default with parens to appease SQLite
+      def column_definition_default_sql(sql, column)
+        sql << " DEFAULT (#{literal(column[:default])})" if column.include?(:default)
+      end
+    
       # Array of PRAGMA SQL statements based on the Database options that should be applied to
       # new connections.
       def connection_pragmas
@@ -268,15 +285,13 @@ module Sequel
         end
 
         begin
-          if foreign_keys
-            metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
-              c = cols.find {|co| co[:name] == row[:from] } or next
-              c[:table] = row[:table]
-              c[:key] = row[:to]
-              c[:on_update] = on_delete_sql_to_sym(row[:on_update])
-              c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
-              # is there any way to get deferrable status?
-            end
+          metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
+            c = cols.find {|co| co[:name] == row[:from] } or next
+            c[:table] = row[:table]
+            c[:key] = row[:to]
+            c[:on_update] = on_delete_sql_to_sym(row[:on_update])
+            c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
+            # is there any way to get deferrable status?
           end
         rescue Sequel::DatabaseError
           # Doesn't work correctly on some versions of JDBC SQLite,
@@ -551,6 +566,16 @@ module Sequel
         sql << BLOB_START << v.unpack(HSTAR).first << APOS
       end
 
+      # Respect the database integer_booleans setting, using 0 or 'f'.
+      def literal_false
+        @db.integer_booleans ? '0' : "'f'"
+      end
+
+      # Respect the database integer_booleans setting, using 1 or 't'.
+      def literal_true
+        @db.integer_booleans ? '1' : "'t'"
+      end
+
       # SQLite does not support the SQL WITH clause
       def select_clause_methods
         SELECT_CLAUSE_METHODS
