@@ -431,6 +431,10 @@ module Sequel
     #
     #   DB[:table].select_hash([:id, :foo], [:name, :bar]) # SELECT * FROM table
     #   # {[1, 3]=>['a', 'c'], [2, 4]=>['b', 'd'], ...}
+    #
+    # When using this method, you must be sure that each expression has an alias
+    # that Sequel can determine.  Usually you can do this by calling the #as method
+    # on the expression and providing an alias.
     def select_hash(key_column, value_column)
       if key_column.is_a?(Array)
         if value_column.is_a?(Array)
@@ -461,6 +465,10 @@ module Sequel
     #
     #   DB[:table].select_map([:id, :name]) # SELECT id, name FROM table
     #   # => [[1, 'A'], [2, 'B'], [3, 'C'], ...]
+    #
+    # If you provide an array of expressions, you must be sure that each entry
+    # in the array has an alias that Sequel can determine.  Usually you can do this
+    # by calling the #as method on the expression and providing an alias.
     def select_map(column=nil, &block)
       _select_map(column, false, &block)
     end
@@ -478,6 +486,10 @@ module Sequel
     #
     #   DB[:table].select_order_map([:id, :name]) # SELECT id, name FROM table ORDER BY id, name
     #   # => [[1, 'A'], [2, 'B'], [3, 'C'], ...]
+    #
+    # If you provide an array of expressions, you must be sure that each entry
+    # in the array has an alias that Sequel can determine.  Usually you can do this
+    # by calling the #as method on the expression and providing an alias.
     def select_order_map(column=nil, &block)
       _select_map(column, true, &block)
     end
@@ -636,17 +648,12 @@ module Sequel
     # Internals of +select_map+ and +select_order_map+
     def _select_map(column, order, &block)
       ds = naked.ungraphed
-      if column
-        raise(Error, ARG_BLOCK_ERROR_MSG) if block
-        columns = Array(column)
-        select_cols = order ? columns.map{|c| c.is_a?(SQL::OrderedExpression) ? c.expression : c} : columns
-        ds = ds.select(*select_cols)
-        ds = ds.order(*columns.map{|c| unaliased_identifier(c)}) if order
-      else
-        ds = ds.select(&block)
-        ds = ds.order(&block) if order
-      end
-      if column.is_a?(Array)
+      columns = Array(column)
+      virtual_row_columns(columns, block)
+      select_cols = order ? columns.map{|c| c.is_a?(SQL::OrderedExpression) ? c.expression : c} : columns
+      ds = ds.select(*select_cols)
+      ds = ds.order(*columns.map{|c| unaliased_identifier(c)}) if order
+      if column.is_a?(Array) || (columns.length > 1)
         ds._select_map_multiple(select_cols.map{|c| hash_key_symbol(c)})
       else
         ds._select_map_single
@@ -694,6 +701,8 @@ module Sequel
         hash_key_symbol(s.column)
       when SQL::AliasedExpression
         hash_key_symbol(s.aliaz)
+      when String
+        s.to_sym
       else
         raise(Error, "#{s.inspect} is not supported, should be a Symbol, String, SQL::Identifier, SQL::QualifiedIdentifier, or SQL::AliasedExpression") 
       end
