@@ -566,6 +566,7 @@ describe "Database#table_exists?" do
   specify "should try to select the first record from the table's dataset" do
     db = Sequel.mock(:fetch=>[Sequel::Error, [], [{:a=>1}]])
     db.table_exists?(:a).should be_false
+    db.sqls.should == ["SELECT NULL FROM a LIMIT 1"]
     db.table_exists?(:b).should be_true
     db.table_exists?(:c).should be_true
   end
@@ -1395,6 +1396,11 @@ describe "Database#server_opts" do
     opts = {:host=>1, :database=>2, :servers=>{:server1=>2}}
     proc{Sequel::Database.new(opts).send(:server_opts, :server1)}.should raise_error(Sequel::Error)
   end
+
+  specify "should return the general opts merged with given opts if given opts is a Hash" do
+    opts = {:host=>1, :database=>2}
+    Sequel::Database.new(opts).send(:server_opts, :host=>2)[:host].should == 2
+  end
 end
 
 describe "Database#add_servers" do
