@@ -102,6 +102,192 @@ shared_examples_for "eager limit strategies" do
   end
 end
 
+shared_examples_for "filtering/excluding by associations" do
+  specify "should work correctly when filtering by associations" do
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+    
+    @Artist.filter(:albums=>@album).all.should == [@artist]
+    @Artist.filter(:first_album=>@album).all.should == [@artist]
+    @Album.filter(:artist=>@artist).all.should == [@album]
+    @Album.filter(:tags=>@tag).all.should == [@album]
+    @Album.filter(:alias_tags=>@tag).all.should == [@album]
+    @Tag.filter(:albums=>@album).all.should == [@tag]
+    @Album.filter(:artist=>@artist, :tags=>@tag).all.should == [@album]
+    @artist.albums_dataset.filter(:tags=>@tag).all.should == [@album]
+  end
+
+  specify "should work correctly when excluding by associations" do
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+    album, artist, tag = @pr.call
+
+    @Artist.exclude(:albums=>@album).all.should == [artist]
+    @Artist.exclude(:first_album=>@album).all.should == [artist]
+    @Album.exclude(:artist=>@artist).all.should == [album]
+    @Album.exclude(:tags=>@tag).all.should == [album]
+    @Album.exclude(:alias_tags=>@tag).all.should == [album]
+    @Tag.exclude(:albums=>@album).all.should == [tag]
+    @Album.exclude(:artist=>@artist, :tags=>@tag).all.should == [album]
+  end
+  
+  specify "should work correctly when filtering by multiple associations" do
+    album, artist, tag = @pr.call
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+    
+    @Artist.filter(:albums=>[@album, album]).all.should == [@artist]
+    @Artist.filter(:first_album=>[@album, album]).all.should == [@artist]
+    @Album.filter(:artist=>[@artist, artist]).all.should == [@album]
+    @Album.filter(:tags=>[@tag, tag]).all.should == [@album]
+    @Album.filter(:alias_tags=>[@tag, tag]).all.should == [@album]
+    @Tag.filter(:albums=>[@album, album]).all.should == [@tag]
+    @Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [@album]
+    @artist.albums_dataset.filter(:tags=>[@tag, tag]).all.should == [@album]
+
+    album.add_tag(tag)
+
+    @Artist.filter(:albums=>[@album, album]).all.should == [@artist]
+    @Artist.filter(:first_album=>[@album, album]).all.should == [@artist]
+    @Album.filter(:artist=>[@artist, artist]).all.should == [@album]
+    @Album.filter(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Tag.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
+    @Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [@album]
+
+    album.update(:artist => artist)
+
+    @Artist.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Artist.filter(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Album.filter(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Tag.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
+    @Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+  end
+
+  specify "should work correctly when excluding by multiple associations" do
+    album, artist, tag = @pr.call
+
+    @Artist.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Artist.exclude(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Album.exclude(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.exclude(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.exclude(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Tag.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
+    @Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
+
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+
+    @Artist.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [artist]
+    @Artist.exclude(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [artist]
+    @Album.exclude(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [album]
+    @Album.exclude(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
+    @Album.exclude(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
+    @Tag.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [tag]
+    @Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
+
+    album.add_tag(tag)
+
+    @Artist.exclude(:albums=>[@album, album]).all.should == [artist]
+    @Artist.exclude(:first_album=>[@album, album]).all.should == [artist]
+    @Album.exclude(:artist=>[@artist, artist]).all.should == [album]
+    @Album.exclude(:tags=>[@tag, tag]).all.should == []
+    @Album.exclude(:alias_tags=>[@tag, tag]).all.should == []
+    @Tag.exclude(:albums=>[@album, album]).all.should == []
+    @Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [album]
+
+    album.update(:artist => artist)
+
+    @Artist.exclude(:albums=>[@album, album]).all.should == []
+    @Artist.exclude(:first_album=>[@album, album]).all.should == []
+    @Album.exclude(:artist=>[@artist, artist]).all.should == []
+    @Album.exclude(:tags=>[@tag, tag]).all.should == []
+    @Album.exclude(:alias_tags=>[@tag, tag]).all.should == []
+    @Tag.exclude(:albums=>[@album, album]).all.should == []
+    @Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == []
+  end
+  
+  specify "should work correctly when excluding by associations in regards to NULL values" do
+    @Artist.exclude(:albums=>@album).all.should == [@artist]
+    @Artist.exclude(:first_album=>@album).all.should == [@artist]
+    @Album.exclude(:artist=>@artist).all.should == [@album]
+    @Album.exclude(:tags=>@tag).all.should == [@album]
+    @Album.exclude(:alias_tags=>@tag).all.should == [@album]
+    @Tag.exclude(:albums=>@album).all.should == [@tag]
+    @Album.exclude(:artist=>@artist, :tags=>@tag).all.should == [@album]
+
+    @album.update(:artist => @artist)
+    @artist.albums_dataset.exclude(:tags=>@tag).all.should == [@album]
+  end
+
+  specify "should handle NULL values in join table correctly when filtering/excluding many_to_many associations" do
+    @ins.call
+    @Album.exclude(:tags=>@tag).all.should == [@album]
+    @Album.exclude(:alias_tags=>@tag).all.should == [@album]
+    @album.add_tag(@tag)
+    @Album.filter(:tags=>@tag).all.should == [@album]
+    @Album.filter(:alias_tags=>@tag).all.should == [@album]
+    album, artist, tag = @pr.call
+    @Album.exclude(:tags=>@tag).all.should == [album]
+    @Album.exclude(:alias_tags=>@tag).all.should == [album]
+    @Album.exclude(:tags=>tag).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.exclude(:alias_tags=>tag).all.sort_by{|x| x.pk}.should == [@album, album]
+  end
+
+  specify "should work correctly when filtering by association datasets" do
+    album, artist, tag = @pr.call
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+    album.add_tag(tag)
+    album.update(:artist => artist)
+
+    @Artist.filter(:albums=>@Album).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Artist.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
+    @Artist.filter(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+    @Artist.filter(:first_album=>@Album).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Artist.filter(:first_album=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
+    @Artist.filter(:first_album=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+    @Album.filter(:artist=>@Artist).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| k.qualify(Artist.table_name)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:artist=>@Artist.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+    @Album.filter(:tags=>@Tag).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+    @Album.filter(:alias_tags=>@Tag).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.filter(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:alias_tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+    @Tag.filter(:albums=>@Album).all.sort_by{|x| x.pk}.should == [@tag, tag]
+    @Tag.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [tag]
+    @Tag.filter(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
+  end
+
+  specify "should work correctly when excluding by association datasets" do
+    album, artist, tag = @pr.call
+    @album.update(:artist => @artist)
+    @album.add_tag(@tag)
+    album.add_tag(tag)
+    album.update(:artist => artist)
+
+    @Artist.exclude(:albums=>@Album).all.sort_by{|x| x.pk}.should == []
+    @Artist.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@artist]
+    @Artist.exclude(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@artist, artist]
+    @Album.exclude(:artist=>@Artist).all.sort_by{|x| x.pk}.should == []
+    @Album.exclude(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| k.qualify(Artist.table_name)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:artist=>@Artist.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.exclude(:tags=>@Tag).all.sort_by{|x| x.pk}.should == []
+    @Album.exclude(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Album.exclude(:alias_tags=>@Tag).all.sort_by{|x| x.pk}.should == []
+    @Album.exclude(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:alias_tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
+    @Tag.exclude(:albums=>@Album).all.sort_by{|x| x.pk}.should == []
+    @Tag.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@tag]
+    @Tag.exclude(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@tag, tag]
+  end
+end
+
 shared_examples_for "regular and composite key associations" do  
   specify "should return no objects if none are associated" do
     @album.artist.should == nil
@@ -145,190 +331,6 @@ shared_examples_for "regular and composite key associations" do
     @tag.albums.should == [@album]
   end
 
-  specify "should work correctly when filtering by associations" do
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-    
-    Artist.filter(:albums=>@album).all.should == [@artist]
-    Artist.filter(:first_album=>@album).all.should == [@artist]
-    Album.filter(:artist=>@artist).all.should == [@album]
-    Album.filter(:tags=>@tag).all.should == [@album]
-    Album.filter(:alias_tags=>@tag).all.should == [@album]
-    Tag.filter(:albums=>@album).all.should == [@tag]
-    Album.filter(:artist=>@artist, :tags=>@tag).all.should == [@album]
-    @artist.albums_dataset.filter(:tags=>@tag).all.should == [@album]
-  end
-
-  specify "should work correctly when excluding by associations" do
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-    album, artist, tag = @pr.call
-
-    Artist.exclude(:albums=>@album).all.should == [artist]
-    Artist.exclude(:first_album=>@album).all.should == [artist]
-    Album.exclude(:artist=>@artist).all.should == [album]
-    Album.exclude(:tags=>@tag).all.should == [album]
-    Album.exclude(:alias_tags=>@tag).all.should == [album]
-    Tag.exclude(:albums=>@album).all.should == [tag]
-    Album.exclude(:artist=>@artist, :tags=>@tag).all.should == [album]
-  end
-  
-  specify "should work correctly when filtering by multiple associations" do
-    album, artist, tag = @pr.call
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-    
-    Artist.filter(:albums=>[@album, album]).all.should == [@artist]
-    Artist.filter(:first_album=>[@album, album]).all.should == [@artist]
-    Album.filter(:artist=>[@artist, artist]).all.should == [@album]
-    Album.filter(:tags=>[@tag, tag]).all.should == [@album]
-    Album.filter(:alias_tags=>[@tag, tag]).all.should == [@album]
-    Tag.filter(:albums=>[@album, album]).all.should == [@tag]
-    Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [@album]
-    @artist.albums_dataset.filter(:tags=>[@tag, tag]).all.should == [@album]
-
-    album.add_tag(tag)
-
-    Artist.filter(:albums=>[@album, album]).all.should == [@artist]
-    Artist.filter(:first_album=>[@album, album]).all.should == [@artist]
-    Album.filter(:artist=>[@artist, artist]).all.should == [@album]
-    Album.filter(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Tag.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
-    Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [@album]
-
-    album.update(:artist => artist)
-
-    Artist.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Artist.filter(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Album.filter(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Tag.filter(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
-    Album.filter(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-  end
-
-  specify "should work correctly when excluding by multiple associations" do
-    album, artist, tag = @pr.call
-
-    Artist.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Artist.exclude(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Album.exclude(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.exclude(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.exclude(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-    Tag.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [@tag, tag]
-    Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [@album, album]
-
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-
-    Artist.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [artist]
-    Artist.exclude(:first_album=>[@album, album]).all.sort_by{|x| x.pk}.should == [artist]
-    Album.exclude(:artist=>[@artist, artist]).all.sort_by{|x| x.pk}.should == [album]
-    Album.exclude(:tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
-    Album.exclude(:alias_tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
-    Tag.exclude(:albums=>[@album, album]).all.sort_by{|x| x.pk}.should == [tag]
-    Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.sort_by{|x| x.pk}.should == [album]
-
-    album.add_tag(tag)
-
-    Artist.exclude(:albums=>[@album, album]).all.should == [artist]
-    Artist.exclude(:first_album=>[@album, album]).all.should == [artist]
-    Album.exclude(:artist=>[@artist, artist]).all.should == [album]
-    Album.exclude(:tags=>[@tag, tag]).all.should == []
-    Album.exclude(:alias_tags=>[@tag, tag]).all.should == []
-    Tag.exclude(:albums=>[@album, album]).all.should == []
-    Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == [album]
-
-    album.update(:artist => artist)
-
-    Artist.exclude(:albums=>[@album, album]).all.should == []
-    Artist.exclude(:first_album=>[@album, album]).all.should == []
-    Album.exclude(:artist=>[@artist, artist]).all.should == []
-    Album.exclude(:tags=>[@tag, tag]).all.should == []
-    Album.exclude(:alias_tags=>[@tag, tag]).all.should == []
-    Tag.exclude(:albums=>[@album, album]).all.should == []
-    Album.exclude(:artist=>[@artist, artist], :tags=>[@tag, tag]).all.should == []
-  end
-  
-  specify "should work correctly when excluding by associations in regards to NULL values" do
-    Artist.exclude(:albums=>@album).all.should == [@artist]
-    Artist.exclude(:first_album=>@album).all.should == [@artist]
-    Album.exclude(:artist=>@artist).all.should == [@album]
-    Album.exclude(:tags=>@tag).all.should == [@album]
-    Album.exclude(:alias_tags=>@tag).all.should == [@album]
-    Tag.exclude(:albums=>@album).all.should == [@tag]
-    Album.exclude(:artist=>@artist, :tags=>@tag).all.should == [@album]
-
-    @album.update(:artist => @artist)
-    @artist.albums_dataset.exclude(:tags=>@tag).all.should == [@album]
-  end
-
-  specify "should handle NULL values in join table correctly when filtering/excluding many_to_many associations" do
-    @ins.call
-    Album.exclude(:tags=>@tag).all.should == [@album]
-    Album.exclude(:alias_tags=>@tag).all.should == [@album]
-    @album.add_tag(@tag)
-    Album.filter(:tags=>@tag).all.should == [@album]
-    Album.filter(:alias_tags=>@tag).all.should == [@album]
-    album, artist, tag = @pr.call
-    Album.exclude(:tags=>@tag).all.should == [album]
-    Album.exclude(:alias_tags=>@tag).all.should == [album]
-    Album.exclude(:tags=>tag).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.exclude(:alias_tags=>tag).all.sort_by{|x| x.pk}.should == [@album, album]
-  end
-
-  specify "should work correctly when filtering by association datasets" do
-    album, artist, tag = @pr.call
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-    album.add_tag(tag)
-    album.update(:artist => artist)
-
-    Artist.filter(:albums=>Album.dataset).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Artist.filter(:albums=>Album.dataset.filter(Array(Album.primary_key).zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
-    Artist.filter(:albums=>Album.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-    Artist.filter(:first_album=>Album.dataset).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Artist.filter(:first_album=>Album.dataset.filter(Array(Album.primary_key).zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
-    Artist.filter(:first_album=>Album.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-    Album.filter(:artist=>Artist.dataset).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:artist=>Artist.dataset.filter(Array(Artist.primary_key).zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [album]
-    Album.filter(:artist=>Artist.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-    Album.filter(:tags=>Tag.dataset).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:tags=>Tag.dataset.filter(Array(Tag.primary_key).zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
-    Album.filter(:tags=>Tag.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-    Album.filter(:alias_tags=>Tag.dataset).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.filter(:alias_tags=>Tag.dataset.filter(Array(Tag.primary_key).zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
-    Album.filter(:alias_tags=>Tag.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-    Tag.filter(:albums=>Album.dataset).all.sort_by{|x| x.pk}.should == [@tag, tag]
-    Tag.filter(:albums=>Album.dataset.filter(Array(Album.primary_key).zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [tag]
-    Tag.filter(:albums=>Album.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
-  end
-
-  specify "should work correctly when excluding by association datasets" do
-    album, artist, tag = @pr.call
-    @album.update(:artist => @artist)
-    @album.add_tag(@tag)
-    album.add_tag(tag)
-    album.update(:artist => artist)
-
-    Artist.exclude(:albums=>Album.dataset).all.sort_by{|x| x.pk}.should == []
-    Artist.exclude(:albums=>Album.dataset.filter(Array(Album.primary_key).zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@artist]
-    Artist.exclude(:albums=>Album.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    Album.exclude(:artist=>Artist.dataset).all.sort_by{|x| x.pk}.should == []
-    Album.exclude(:artist=>Artist.dataset.filter(Array(Artist.primary_key).zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [@album]
-    Album.exclude(:artist=>Artist.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.exclude(:tags=>Tag.dataset).all.sort_by{|x| x.pk}.should == []
-    Album.exclude(:tags=>Tag.dataset.filter(Array(Tag.primary_key).zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
-    Album.exclude(:tags=>Tag.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
-    Album.exclude(:alias_tags=>Tag.dataset).all.sort_by{|x| x.pk}.should == []
-    Album.exclude(:alias_tags=>Tag.dataset.filter(Array(Tag.primary_key).zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
-    Album.exclude(:alias_tags=>Tag.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
-    Tag.exclude(:albums=>Album.dataset).all.sort_by{|x| x.pk}.should == []
-    Tag.exclude(:albums=>Album.dataset.filter(Array(Album.primary_key).zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@tag]
-    Tag.exclude(:albums=>Album.dataset.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@tag, tag]
-  end
-
   specify "should have working dataset associations" do
     album, artist, tag = @pr.call
 
@@ -442,6 +444,30 @@ shared_examples_for "regular and composite key associations" do
     a.first.albums.first.artist.should == @artist
   end
   
+  describe "when filtering/excluding by associations" do
+    before do
+      @Artist = Artist.dataset
+      @Album = Album.dataset
+      @Tag = Tag.dataset
+    end
+
+    it_should_behave_like "filtering/excluding by associations"
+  end
+
+  describe "when filtering/excluding by associations when joining" do
+    def self_join(c)
+      c.join(c.table_name.as(:b), Array(c.primary_key).zip(Array(c.primary_key))).select_all(c.table_name)
+    end
+
+    before do
+      @Artist = self_join(Artist)
+      @Album = self_join(Album)
+      @Tag = self_join(Tag)
+    end
+
+    it_should_behave_like "filtering/excluding by associations"
+  end
+
   describe "with no :eager_limit_strategy" do
     it_should_behave_like "eager limit strategies"
   end
@@ -838,3 +864,101 @@ describe "Sequel::Model Composite Key Associations" do
     proc{@artist.remove_album(@album)}.should raise_error(Sequel::Error)
   end
 end
+
+describe "Sequel::Model Associations with clashing column names" do
+  before(:all) do
+    @db = INTEGRATION_DB
+    [:bars_foos, :bars, :foos].each{|t| @db.drop_table(t) rescue nil}
+    @db.create_table(:foos) do
+      primary_key :id
+      Integer :object_id
+    end
+    @db.create_table(:bars) do
+      primary_key :id
+      Integer :object_id
+    end
+    @db.create_table(:bars_foos) do
+      Integer :foo_id
+      Integer :object_id
+      primary_key [:foo_id, :object_id]
+    end
+  end
+  before do
+    [:bars_foos, :bars, :foos].each{|t| @db[t].delete}
+    @Foo = Class.new(Sequel::Model(:foos))
+    @Bar = Class.new(Sequel::Model(:bars))
+    @Foo.def_column_alias(:obj_id, :object_id)
+    @Bar.def_column_alias(:obj_id, :object_id)
+    @Foo.one_to_many :bars, :primary_key=>:obj_id, :primary_key_column=>:object_id, :key=>:object_id, :key_method=>:obj_id, :eager_loader_key=>:object_id, :class=>@Bar
+    @Foo.one_to_one :bar, :primary_key=>:obj_id, :primary_key_column=>:object_id, :key=>:object_id, :key_method=>:obj_id, :eager_loader_key=>:object_id, :class=>@Bar
+    @Bar.many_to_one :foo, :key=>:obj_id, :key_column=>:object_id, :primary_key=>:object_id, :primary_key_method=>:obj_id, :class=>@Foo
+    @Foo.many_to_many :mtmbars, :join_table=>:bars_foos, :left_primary_key=>:obj_id, :left_primary_key_column=>:object_id, :right_primary_key=>:object_id, :right_primary_key_method=>:obj_id, :left_key=>:foo_id, :right_key=>:object_id, :eager_loader_key=>:object_id, :class=>@Bar
+    @Bar.many_to_many :mtmfoos, :join_table=>:bars_foos, :left_primary_key=>:obj_id, :left_primary_key_column=>:object_id, :right_primary_key=>:object_id, :right_primary_key_method=>:obj_id, :left_key=>:object_id, :right_key=>:foo_id, :eager_loader_key=>:object_id, :class=>@Foo
+    @foo = @Foo.create(:obj_id=>2)
+    @bar = @Bar.create(:obj_id=>2)
+    @Foo.db[:bars_foos].insert(2, 2)
+  end
+  after(:all) do
+    @db.drop_table(:bars_foos, :bars, :foos)
+  end
+
+  it "should have working regular association methods" do
+    @Bar.first.foo.should == @foo
+    @Foo.first.bars.should == [@bar]
+    @Foo.first.bar.should == @bar
+    @Foo.first.mtmbars.should == [@bar]
+    @Bar.first.mtmfoos.should == [@foo]
+  end
+
+  it "should have working eager loading methods" do
+    @Bar.eager(:foo).all.map{|o| [o, o.foo]}.should == [[@bar, @foo]]
+    @Foo.eager(:bars).all.map{|o| [o, o.bars]}.should == [[@foo, [@bar]]]
+    @Foo.eager(:bar).all.map{|o| [o, o.bar]}.should == [[@foo, @bar]]
+    @Foo.eager(:mtmbars).all.map{|o| [o, o.mtmbars]}.should == [[@foo, [@bar]]]
+    @Bar.eager(:mtmfoos).all.map{|o| [o, o.mtmfoos]}.should == [[@bar, [@foo]]]
+  end
+
+  it "should have working eager graphing methods" do
+    @Bar.eager_graph(:foo).all.map{|o| [o, o.foo]}.should == [[@bar, @foo]]
+    @Foo.eager_graph(:bars).all.map{|o| [o, o.bars]}.should == [[@foo, [@bar]]]
+    @Foo.eager_graph(:bar).all.map{|o| [o, o.bar]}.should == [[@foo, @bar]]
+    @Foo.eager_graph(:mtmbars).all.map{|o| [o, o.mtmbars]}.should == [[@foo, [@bar]]]
+    @Bar.eager_graph(:mtmfoos).all.map{|o| [o, o.mtmfoos]}.should == [[@bar, [@foo]]]
+  end
+
+  it "should have working modification methods" do
+    b = @Bar.create(:obj_id=>3)
+    f = @Foo.create(:obj_id=>3)
+
+    @bar.foo = f
+    @bar.obj_id.should == 3
+    @foo.bar = @bar
+    @bar.obj_id.should == 2
+
+    @foo.add_bar(b)
+    @foo.bars.sort_by{|x| x.obj_id}.should == [@bar, b]
+    @foo.remove_bar(b)
+    @foo.bars.should == [@bar]
+    @foo.remove_all_bars
+    @foo.bars.should == []
+
+    @bar.refresh.update(:obj_id=>2)
+    b.refresh.update(:obj_id=>3)
+    @foo.mtmbars.should == [@bar]
+    @foo.remove_all_mtmbars
+    @foo.mtmbars.should == []
+    @foo.add_mtmbar(b)
+    @foo.mtmbars.should == [b]
+    @foo.remove_mtmbar(b)
+    @foo.mtmbars.should == []
+
+    @bar.add_mtmfoo(f)
+    @bar.mtmfoos.should == [f]
+    @bar.remove_all_mtmfoos
+    @bar.mtmfoos.should == []
+    @bar.add_mtmfoo(f)
+    @bar.mtmfoos.should == [f]
+    @bar.remove_mtmfoo(f)
+    @bar.mtmfoos.should == []
+  end
+end 
