@@ -81,7 +81,7 @@ describe "Simple Dataset operations" do
     @ds.all.should == [{:id=>1, :number=>11}]
   end
   
-  cspecify "should have update return the number of matched rows", [:mysql, :mysql], [:do, :mysql], [:mysql2], [:ado] do
+  cspecify "should have update return the number of matched rows", [:do, :mysql], [:ado] do
     @ds.update(:number=>:number).should == 1
     @ds.filter(:id=>1).update(:number=>:number).should == 1
     @ds.filter(:id=>2).update(:number=>:number).should == 0
@@ -674,6 +674,12 @@ describe Sequel::SQL::Constants do
     @ds.insert(:ts=>Sequel::CURRENT_TIMESTAMP)
     (Time.now - @c[@ds.get(:ts)]).should be_within(2).of(0)
   end
+
+  cspecify "should have working CURRENT_TIMESTAMP when used as a column default", [:jdbc, :sqlite] do
+    @db.create_table!(:constants){DateTime :ts, :default=>Sequel::CURRENT_TIMESTAMP}
+    @ds.insert
+    (Time.now - @c[@ds.get(:ts)]).should be_within(2).of(0)
+  end
 end
 
 describe "Sequel::Dataset#import and #multi_insert" do
@@ -1178,11 +1184,42 @@ describe "Sequel::Dataset DSL support" do
   specify "should work empty arrays with nulls" do
     @ds.insert(nil, nil)
     @ds.filter(:a=>[]).all.should == []
-    @ds.exclude(:a=>[]).all.should == [{:a=>nil, :b=>nil}]
+    @ds.exclude(:a=>[]).all.should == []
     @ds.filter([:a, :b]=>[]).all.should == []
-    @ds.exclude([:a, :b]=>[]).all.should == [{:a=>nil, :b=>nil}]
+    @ds.exclude([:a, :b]=>[]).all.should == []
+
+    unless Sequel.guarded?(:mssql, :oracle, :db2)
+      # MSSQL doesn't like boolean results in the select list
+      pr = proc{|r| r.is_a?(Integer) ? (r != 0) : r}
+      pr[@ds.get({:a=>[]}.sql_expr)].should == nil
+      pr[@ds.get(~({:a=>[]}).sql_expr)].should == nil
+      pr[@ds.get({[:a, :b]=>[]}.sql_expr)].should == nil
+      pr[@ds.get(~({[:a, :b]=>[]}).sql_expr)].should == nil
+    end
   end
   
+  specify "should work empty arrays with nulls and Sequel.empty_array_null_handling = true" do
+    begin
+      Sequel.empty_array_handle_nulls = false
+      @ds.insert(nil, nil)
+      @ds.filter(:a=>[]).all.should == []
+      @ds.exclude(:a=>[]).all.should == [{:a=>nil, :b=>nil}]
+      @ds.filter([:a, :b]=>[]).all.should == []
+      @ds.exclude([:a, :b]=>[]).all.should == [{:a=>nil, :b=>nil}]
+
+      unless Sequel.guarded?(:mssql, :oracle, :db2)
+        # MSSQL doesn't like boolean results in the select list
+        pr = proc{|r| r.is_a?(Integer) ? (r != 0) : r}
+        pr[@ds.get({:a=>[]}.sql_expr)].should == false
+        pr[@ds.get(~({:a=>[]}).sql_expr)].should == true
+        pr[@ds.get({[:a, :b]=>[]}.sql_expr)].should == false
+        pr[@ds.get(~({[:a, :b]=>[]}).sql_expr)].should == true
+      end
+    ensure
+      Sequel.empty_array_handle_nulls = true
+    end
+  end
+
   it "should work multiple conditions" do
     @ds.insert(20, 10)
     @ds.filter(:a=>20, :b=>10).all.should == [{:a=>20, :b=>10}]
@@ -1320,7 +1357,7 @@ describe "Dataset identifier methods" do
     @db.drop_table(:a)
   end
   
-  cspecify "#identifier_output_method should change how identifiers are output", [:mysql2], [:swift] do
+  cspecify "#identifier_output_method should change how identifiers are output", [:swift] do
     @ds.identifier_output_method = :upcase
     @ds.first.should == {:AB=>1}
     @ds.identifier_output_method = :uprev
